<?php

//Custom Donations Shortcode Class.
class CustomDonations_Shortcode {

    /**
     * @var CustomDonations_Helper
     */
    private $helper;
    private $memberid_enabled, $memberid_field; //fields set in settings.
    public $account, $form, $allocation, $mode, $memberId, /*$paymentVersion,*/ $interval, $useReCaptcha, $shortcodeDebug;
    public $baseUrl = 'https://api.customdonations.com';
    public $apiVersion = 'v1';

    public function __construct($atts) {
        $this->helper = new CustomDonations_Helper();
        $helper = $this->helper;
        extract($atts); //convert the array keys into php variables with the same name.
        $opts = get_option('customdonations_options'); //get settings from the database
        $config_acctid = !empty($opts['customdonations_acctid']) ? $opts['customdonations_acctid'] : null; //is there an account id configured in the DB?
        //$config_paymentver = !empty($opts['customdonations_paymentver']) && is_numeric($opts['customdonations_paymentver']) ? (int) $opts['customdonations_paymentver'] : 1;
        $this->memberid_enabled = $helper::VarToBool($opts['customdonations_memberid_enabled']);
        $this->memberid_field = !empty($opts['customdonations_memberid_field']) ? $opts['customdonations_memberid_field'] : null; //what field should be used for logged-in users?
        $this->allocation = $helper::VarToBool($allocation) ? esc_attr($allocation) : null; //optional
        $this->form = !empty($form) ? esc_attr($form) : null; //definitely required - not good if missing!
        $this->account = !empty($account) ? $account : $config_acctid; //if value for account is changed in shortcode, use that - otherwise go with the value specified in the config.
        $this->memberId = $this->memberid_enabled ? $this->fill_memberid_field() : null;  //if we aren't wanting to fill memberId for logged-in users, use null as the value
        $this->mode = ($mode === 'live' || $mode === 'test') ? esc_attr($mode) : null; //there is an issue if mode isn't provided.
        //$this->paymentVersion = !empty($paymentversion) && $paymentversion > 0 && $paymentversion <= 2 ? $paymentversion : $config_paymentver; //1: for modal payment window. 2: for inline payment with Google/Apple Pay options (US CLIENTS ONLY!)
        $this->interval = $helper::VarToBool($interval) ? trim(strtolower($interval)) : null; // once, monthly, quarterly, yearly, or empty.
        $this->useReCaptcha = $helper::VarToBool($userecaptcha);
        $this->shortcodeDebug = $helper::VarToBool($shortcodedebug);
    }

    /**
     * Checks to see if there are any missing/invalid field values.
     * @return string|boolean Returns error HTML if fields missing, returns false if there are no missing fields.
     */
    private function missing_fields_exist() {
        $txtResult = '';
        if (is_null($this->form)) {
            $txtResult .= sprintf('<p style="color:red;">Value for <strong>%s</strong> is missing.</p>', 'form');
        }
        if (is_null($this->account)) {
            $txtResult .= sprintf('<p style="color:red;">Value for <strong>%s</strong> is missing.</p>', 'account');
        }
        if (is_null($this->mode)) {
            $txtResult .= sprintf('<p style="color:red;">Value for <strong>%s</strong> is missing or invalid.</p>', 'mode');
        }
        return strlen($txtResult) > 0 ? $txtResult : false;
    }

    /**
     * Compile the final result of shortcode, if there are missing fields - outputs error message HTML.
     * @return string output HTML and Javascript.
     */
    public function print_script() {
        $has_missing_fields = $this->missing_fields_exist();
        return !boolval($has_missing_fields) ? $this->build_form_embed() : $has_missing_fields;
    }

    /**
     * Returns form embed code, and calls function to add the form builder and shortcode javascripts.
     * @return string Form embed code
     */
    private function build_form_embed() {
        $embedDiv = '<div id="CustomDonationsForm"></div>';
        $this->enqueue_customdonations_scripts_js();
        return $embedDiv;
    }

    /**
     * Add the form builder and shortcode javascripts.
     */
    function enqueue_customdonations_scripts_js() {
        $dateTime = new DateTime();
        //$frm_builder_script_url = "{$this->baseUrl}/{$this->apiVersion}/js/form-builder.min.js";
        $cacheBust = $dateTime->getTimestamp();
        //wp_enqueue_script('customdonations-form-builder', $frm_builder_script_url, '', $cacheBust, true);
        wp_enqueue_script('customdonations-shortcode', plugins_url('../js/cd-shortcode.js', __FILE__), '', $cacheBust, true);
        $data = [
            'account' => $this->account,
            'form' => $this->form,
            'allocation' => $this->allocation, //will be automatically set to null, if not provided in shortcode.
            'memberId' => $this->memberId, //If someone is logged-in, and the option is enabled fill memberId dynamically. Otherwise, this will be null.
            'apiVersion' => $this->apiVersion,
            'baseUrl' => $this->baseUrl,
            'mode' => $this->mode,
            //'paymentVersion' => $this->paymentVersion,
            'interval' => $this->interval,
            'useReCaptcha' => $this->useReCaptcha,
            'shortcodeDebug' => $this->shortcodeDebug
        ];
        wp_localize_script('customdonations-shortcode', 'shortcode_data', $data); //PASS FORM DATA TO THE EMBED SCRIPT.
    }

    /**
     * Gets the appropriate value for the memberId field of the customdonations form
     * @return null|string memberId for the CustomDonations embed javascript.
     */
    private function fill_memberid_field() {
        $user_current = wp_get_current_user();
        $return_val = null; //return null by default.
		if ($this->memberid_field == "id")
			$this->memberid_field = "ID";
        if (get_class($user_current) == "WP_User" && !empty($this->memberid_field)) { //we have a logged-in user.
            $return_val = esc_attr($user_current->get($this->memberid_field)); //get the desired user field value.
        }
        $this->memberId = $return_val;
        return $return_val;
    }

}

/**
 * Hook for the shortcode function
 * @param array $atts - values provided in the Shortcode.
 * @return string Output HTML for either errors or the CustomDonationsForm div.
 */
function customdonations_shortcode_func($atts) {
    $shortcode = new CustomDonations_Shortcode($atts); //create new shortcode object
    return $shortcode->print_script(); //do everything to process the shortcode and print the divs or the errors.
}

add_shortcode('CustomDonations', 'customdonations_shortcode_func');
add_shortcode('customdonations', 'customdonations_shortcode_func'); //make either Pascal or lower case acceptable for processing.
